//=========================================================
//  MusE
//  Linux Music Editor
//  $Id: midithread.h,v 1.12 2002/11/29 14:24:51 muse Exp $
//
//  (C) Copyright 2001 Werner Schweer (ws@seh.de)
//=========================================================

#ifndef __SEQTHREAD_H__
#define __SEQTHREAD_H__

#include <qobject.h>
#include "thread.h"

class Part;
class Event;
class MidiPlayEvent;
class MidiEvent;
class Track;
class MidiPort;
class MidiDevice;
class EventList;
class MTC;
struct MidiThreadPrivate;
class SynthI;
class MidiInstrument;

//---------------------------------------------------------
//   MidiMsgId
//    this are the messages send from the GUI thread to
//    the midi thread
//---------------------------------------------------------

enum MidiMsgId {
      SEQM_ADD_TRACK, SEQM_REMOVE_TRACK, SEQM_CHANGE_TRACK, SEQM_MOVE_TRACK,
      SEQM_ADD_PART, SEQM_REMOVE_PART, SEQM_CHANGE_PART,
      SEQM_ADD_EVENT, SEQM_REMOVE_EVENT, SEQM_CHANGE_EVENT,
      SEQM_ADD_TEMPO, SEQM_SET_TEMPO, SEQM_REMOVE_TEMPO, SEQM_ADD_SIG,
      SEQM_SET_GLOBAL_TEMPO,
      SEQM_UNDO, SEQM_REDO,
      SEQM_RESET_DEVICES, SEQM_INIT_DEVICES, SEQM_PANIC,
      SEQM_MIDI_LOCAL_OFF,
      SEQM_MIDI_CTRL, SEQM_MIDI_MASTER_VOLUME,
      SEQM_SET_MIDI_DEVICE,
      SEQM_PLAY_MIDI_EVENT,
      SEQM_SET_MIXDOWN,
      SEQM_SET_RTC_TICKS,
      SEQM_SEEK, SEQM_PLAY,
      SEQM_SCAN_ALSA_MIDI_PORTS,
      SEQM_REMOVE_SYNTHI, SEQM_ADD_SYNTHI,
      MIDI_ADD_SYNTHI, MIDI_SHOW_INSTR_GUI,
      };

//---------------------------------------------------------
//   MidiMsg
//---------------------------------------------------------

struct MidiMsg : public ThreadMsg {
      void* p1;
      void* p2;
      void* p3;

      char port;
      char channel;
      char ctrl;

      int a;
      int b;
      int c;
      };

//---------------------------------------------------------
//   MidiThread
//---------------------------------------------------------

class MidiThread : public QObject, public Thread {
      Q_OBJECT;

      QTimer* heartBeatTimer;       // controls gui refreshes
      MidiThreadPrivate* data;

      virtual void processMsg(const ThreadMsg*);
      virtual void defaultTick();

      void updatePollFd();
      bool setRtcTicks();
      int recTimeStamp() const;
      void recordStop();
      void sendLocalOff();
      bool filterEvent(const MidiPlayEvent* event, int type, bool thru);
      void mtcSyncMsg(const MTC& mtc, bool seekFlag);
      void statePlay();
      static void midiTick(void* p, void*);
      void initDevices();

   public slots:
      void seqSignal(int fd);
      void resetDevices();
      void msgInitDevices();
      void localOff();
      void heartBeat();

   signals:
      void midiNote(int, int);

   public:
      MidiThread(int prio, const char* name);
      ~MidiThread();
      void start();
      virtual void threadStop();
      virtual void threadStart(void*);

      bool sendMessage(const MidiMsg* m, bool doUndo);

      bool isPlaying() const;
      void setHeartBeat();

      //-----------------------------------------
      //   message interface
      //-----------------------------------------

      void msgSeek(int val);
      void msgPlay(bool val);

      void msgAddTrack(Track* track, bool u = true);
      void msgRemoveTracks();
      void msgChangeTrack(Track* oldTrack, Track* newTrack, bool u = true);
      void msgMoveTrack(int idx1, int dx2, bool u = true);
      void msgAddPart(Part*, bool u = true);
      bool msgRemoveParts();
      void msgRemovePart(Part*, bool u = true);
      void msgChangePart(Part* oldPart, Part* newPart, bool u = true);
      void msgAddEvent(Event*, Part*, bool u = true);
      void msgDeleteEvent(Event*, Part*, bool u = true);
      void msgChangeEvent(Event*, Event*, Part*, bool u = true);
      void msgScanAlsaMidiPorts();
      void msgAddTempo(int tick, int tempo, bool doUndoFlag = true);
      void msgSetTempo(int tick, int tempo, bool doUndoFlag = true);
      void msgSetGlobalTempo(int val);
      void msgDeleteTempo(int tick, int tempo, bool doUndoFlag = true);
      void msgAddSig(int tick, int z, int n, bool doUndoFlag = true);
      void msgAddSynthI(SynthI* synth);
      void msgShowInstrumentGui(MidiInstrument*, bool);
      void msgPanic();

      void undo();
      void redo();
      void ctrlChanged(int port, int channel, int ctrl, int val);
      void masterVolChanged(int val);

      void setMidiDevice(MidiPort*, MidiDevice*);
      void playMidiEvent(MidiPlayEvent* event);
      void rescanAlsaPorts();

      void eventReceived(MidiPlayEvent* event);
      void eventReceived(int port, unsigned char chan, unsigned char c1, unsigned c2);

      void mmcInput(const unsigned char* p, int n);
      void mtcInputQuarter(int, unsigned char c);
      void mtcInputFull(const unsigned char* p, int n);
      void nonRealtimeSystemSysex(const unsigned char*, int);
      void realtimeSystemInput(int, int);
      void setSongPosition(int, int);
      void midiPortsChanged();
      };

extern MidiThread* midiThread;
#endif

